<?php
/**
 * Fonctions utiles au plugin Incarner
 *
 * @plugin     incarner
 * @copyright  2016
 * @author     Michel Bystranowski
 * @licence    GNU/GPL
 */

/* On perd le droit de changer d'auteur comme on veut après 1h sans activité.
	 C'est important parce qu'on garde ce droit même après déconnexion (pour
	 pouvoir tester le site en tant que visiteur anonyme et revenir ensuite
	 facilement au webmestre). */
if (! defined('_INCARNER_DELAI_EXPIRATION')) {
	define('_INCARNER_DELAI_EXPIRATION', 3600);
}

/**
 * Vérifie si une clé d'incarnation est valide
 *
 * @param string $cle : La clé d'incarnation
 *
 * @return false|int : id_auteur si la clé est valide, false sinon
 */
function incarner_cle_valide($cle) {

	include_spip('inc/config');

	if (! $cles = lire_config('incarner/cles')) {
		$cles = array();
	}
	if (! $maj = lire_config('incarner/maj')) {
		$maj = array();
	}

	if ($cles and ($id_auteur = array_search($cle, $cles))
			and ((time() - $maj[$id_auteur]) < _INCARNER_DELAI_EXPIRATION)) {
		// remettre a jour le flag maj toutes les minutes au plus
		if ((time() - $maj[$id_auteur]) > 60) {
			$maj[$id_auteur] = time();
			ecrire_config('incarner/maj', $maj);
		}

		return $id_auteur;
	} else {
		return false;
	}
}

/**
 * Renouveler la clé d'incarnation
 *
 * Si on en a pas déjà une on en crée une nouvelle
 *
 * @return null
 */
function incarner_renouveler_cle() {

	include_spip('inc/config');
	include_spip('inc/cookie');
	include_spip('inc/session');

	$cle_actuelle = (isset($_COOKIE['spip_cle_incarner']) ? $_COOKIE['spip_cle_incarner'] : '');

	if (! $cles = lire_config('incarner/cles')) {
		$cles = array();
	}
	if (! $maj = lire_config('incarner/maj')) {
		$maj = array();
	}

	$nouvelle_cle = urlencode(bin2hex(random_bytes(16)));

	if (!$id_auteur = incarner_cle_valide($cle_actuelle)) {
		// première incarnation ou cle invalide
		$id_auteur = session_get('id_auteur');
	}

	$cles[$id_auteur] = $nouvelle_cle;
	$maj[$id_auteur] = time();

	ecrire_config('incarner/cles', $cles);
	ecrire_config('incarner/maj', $maj);

	spip_setcookie(
		'spip_cle_incarner',
		$nouvelle_cle,
		defined('_INCARNER_COOKIE_PERSISTANT') ? time()+intval(_INCARNER_COOKIE_PERSISTANT) : 0
	);
}

/**
 * Invalider la clé de l'auteur et effacer son cookie
 *
 * @return null
 */
function incarner_invalider_cle() {

	include_spip('inc/config');
	include_spip('inc/cookie');

	$cle_actuelle = (isset($_COOKIE['spip_cle_incarner']) ? $_COOKIE['spip_cle_incarner'] : '');

	if (! $cles = lire_config('incarner/cles')) {
		$cles = array();
	}
	if (! $maj = lire_config('incarner/maj')) {
		$maj = array();
	}

	$index_cle_actuelle = array_search($cle_actuelle, $cles);

	unset($cles[$index_cle_actuelle]);
	unset($maj[$index_cle_actuelle]);
	ecrire_config('incarner/cles', $cles);
	ecrire_config('incarner/maj', $maj);

	spip_setcookie('spip_cle_incarner', '');
}
